/**
 *   common-jbi-runtime.jar - Set of classes used in building JBI Components
 *
 *   Copyright (C) 2007 Gestalt, LLC. All Rights Reserved.
 *   http://www.gestalt-llc.com/
 *
 *   This library is free software; you can redistribute it and/or
 *   modify it under the terms of the GNU Lesser General Public
 *   License version 2.1 as published by the Free Software Foundation.
 *
 *   This library is distributed in the hope that it will be useful,
 *   but WITHOUT ANY WARRANTY; without even the implied warranty of
 *   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 *   Lesser General Public License for more details.
 *
 *   You should have received a copy of the GNU Lesser General Public
 *   License along with this library; if not, write to the Free Software
 *   Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA  02110-1301 USA
 */

package com.gestalt.jbi.component;

import com.gestalt.jbi.component.lifecycle.AbstractComponentLifeCycle;
import com.gestalt.jbi.component.manager.AbstractServiceUnitManager;
import com.gestalt.jbi.component.manager.Endpoint;
import com.gestalt.jbi.component.manager.ServiceUnit;
import com.sun.jbi.internationalization.Messages;
import org.w3c.dom.Document;
import org.w3c.dom.DocumentFragment;

import javax.jbi.component.Component;
import javax.jbi.component.ComponentLifeCycle;
import javax.jbi.component.ServiceUnitManager;
import javax.jbi.messaging.MessageExchange;
import javax.jbi.servicedesc.ServiceEndpoint;
import java.util.logging.Logger;

/**
 * @author rriven
 *
 */
public abstract class AbstractComponent implements Component {
	protected Logger log = Messages.getLogger(AbstractComponent.class);
	
	protected AbstractComponentLifeCycle componentLifeCycle;
	protected AbstractServiceUnitManager serviceUnitManager;
	
	public AbstractComponent() {
		componentLifeCycle = createComponentLifeCycle(this);
		serviceUnitManager = createServiceUnitManager(this);
	}

	/* (non-Javadoc)
	 * @see javax.jbi.component.Component#getLifeCycle()
	 */
	public ComponentLifeCycle getLifeCycle() {
		return componentLifeCycle;
	}
	
	public abstract AbstractComponentLifeCycle createComponentLifeCycle(AbstractComponent component);

	/* (non-Javadoc)
	 * @see javax.jbi.component.Component#getServiceDescription(javax.jbi.servicedesc.ServiceEndpoint)
	 */
	public Document getServiceDescription(ServiceEndpoint serviceEndpoint) {
		for (ServiceUnit serviceUnit : serviceUnitManager.getServiceUnits()) {
			for (Endpoint endpoint : serviceUnit.getEndpoints()) {
				if (endpoint.getServiceName().equals(serviceEndpoint.getServiceName()) &&
					endpoint.getEndpointName().equals(serviceEndpoint.getEndpointName())) {
					return endpoint.getServiceDescription();
				}
			}
		}
		return null;
	}

	/* (non-Javadoc)
	 * @see javax.jbi.component.Component#getServiceUnitManager()
	 */
	public ServiceUnitManager getServiceUnitManager() {
		return serviceUnitManager;
	}
	
	public abstract AbstractServiceUnitManager createServiceUnitManager(AbstractComponent component);

	/* (non-Javadoc)
	 * @see javax.jbi.component.Component#isExchangeWithConsumerOkay(javax.jbi.servicedesc.ServiceEndpoint, javax.jbi.messaging.MessageExchange)
	 */
	public boolean isExchangeWithConsumerOkay(ServiceEndpoint serviceEndpoint,
			MessageExchange messageExchange) {
		return false;
	}

	/* (non-Javadoc)
	 * @see javax.jbi.component.Component#isExchangeWithProviderOkay(javax.jbi.servicedesc.ServiceEndpoint, javax.jbi.messaging.MessageExchange)
	 */
	public boolean isExchangeWithProviderOkay(ServiceEndpoint serviceEndpoint,
			MessageExchange messageExchange) {
		return true;
	}

	/* (non-Javadoc)
	 * @see javax.jbi.component.Component#resolveEndpointReference(org.w3c.dom.DocumentFragment)
	 */
	public ServiceEndpoint resolveEndpointReference(DocumentFragment documentFragment) {
		return null;
	}

}
