/**
 *   common-jbi-runtime.jar - Set of classes used in building JBI Components
 *
 *   Copyright (C) 2007 Gestalt, LLC. All Rights Reserved.
 *   http://www.gestalt-llc.com/
 *
 *   This library is free software; you can redistribute it and/or
 *   modify it under the terms of the GNU Lesser General Public
 *   License version 2.1 as published by the Free Software Foundation.
 *
 *   This library is distributed in the hope that it will be useful,
 *   but WITHOUT ANY WARRANTY; without even the implied warranty of
 *   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 *   Lesser General Public License for more details.
 *
 *   You should have received a copy of the GNU Lesser General Public
 *   License along with this library; if not, write to the Free Software
 *   Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA  02110-1301 USA
 */

package com.gestalt.jbi.component.manager;

import java.util.Map;
import java.util.logging.Logger;

import javax.jbi.messaging.MessageExchange.Role;
import javax.jbi.servicedesc.ServiceEndpoint;
import javax.wsdl.Definition;
import javax.xml.namespace.QName;

import org.w3c.dom.Document;

import com.gestalt.jbi.component.handler.AbstractMessageExchangeHandler;
import com.sun.jbi.internationalization.Messages;

/**
 * @author rriven
 *
 */
public abstract class Endpoint {
    protected Logger log = Messages.getLogger(Endpoint.class);
    protected Messages messages = Messages.getMessages(Endpoint.class);

    protected QName serviceName;
    protected QName interfaceName;
    protected String endpointName;
    protected Role role;
    protected Document serviceDescription;
    protected Definition wsdlDefinition;

    protected ServiceUnit serviceUnit;
    protected ServiceEndpoint activatedEndpoint;

    protected AbstractMessageExchangeHandler processor;

    public Endpoint(QName serviceName, QName interfaceName, String endpointName, Role role, Document serviceDescription, Definition wsdlDefinition, ServiceUnit serviceUnit) {
        this.serviceName = serviceName;
        this.interfaceName = interfaceName;
        this.endpointName = endpointName;
        this.role = role;
        this.serviceDescription = serviceDescription;
        this.wsdlDefinition = wsdlDefinition;
        this.serviceUnit = serviceUnit;
    }

    public abstract Map getOperations();

    public ServiceUnit getServiceUnit() {
        return serviceUnit;
    }

    public QName getServiceName() {
        return serviceName;
    }

    public String getEndpointName() {
        return endpointName;
    }

    public Document getServiceDescription() {
        return serviceDescription;
    }

    public Definition getWsdlDefinition() {
        return wsdlDefinition;
    }

    public abstract void activate() throws Exception;
    
    public abstract void deactivate() throws Exception;

    public AbstractMessageExchangeHandler getProcessor() {
        return processor;
    }

    public abstract AbstractMessageExchangeHandler createConsumerExchangeHandler();

    public abstract AbstractMessageExchangeHandler createProviderExchangeHandler();

    public String toString() {
       String prefix = "svc";
       String nsURI = null;
       String localName = null;
       String sn = "null";

       if (this.serviceName != null) {
          String namePrefix = this.serviceName.getPrefix();
          if (namePrefix != null && namePrefix.length() > 0) {
             prefix = namePrefix;
          }
          nsURI = this.serviceName.getNamespaceURI();
          localName = this.serviceName.getLocalPart();

          sn = prefix + ":" + localName;
       }

       return "<" +
          image (this.role) + " " +
          "service-name=\"" + sn + "\" " +
          "endpoint-name=\"" + this.endpointName + "\"" +
          (nsURI == null ? "" : " xmlns:" + prefix + "=\"" + nsURI + "\"") +
          ">";
    }

    static String image (Role role) {
       if (Role.CONSUMER.equals (role)) {
          return "consumer";
       } else if (Role.PROVIDER.equals (role)) {
          return "provider";
       } else {
          return String.valueOf (role);
       }
    }
}
