package com.gestalt.jbi.component.manager;

import java.util.logging.Level;
import com.sun.jbi.internationalization.Messages;

import javax.jbi.messaging.DeliveryChannel;
import javax.jbi.messaging.MessageExchange.Role;
import javax.jbi.servicedesc.ServiceEndpoint;
import javax.wsdl.Definition;
import javax.xml.namespace.QName;

import org.w3c.dom.Document;

import com.gestalt.jbi.component.handler.AbstractMessageExchangeHandler;
import com.gestalt.jbi.component.lifecycle.AbstractComponentLifeCycle;

public abstract class PooledProviderEndpoint extends Endpoint {
    
    public PooledProviderEndpoint(QName serviceName, QName interfaceName, String endpointName, Role role, Document serviceDescription, Definition wsdlDefinition, ServiceUnit serviceUnit) {
        super(serviceName, interfaceName, endpointName, role, serviceDescription,
                wsdlDefinition, serviceUnit);
    }

    public void activate() throws Exception {
        log.log(Level.INFO, messages.getString("CJBIRT-R00885.activatingEndpoint", this));
        if (Role.PROVIDER.equals(role)) {
            activatedEndpoint = ((AbstractComponentLifeCycle) serviceUnit.getComponent().getLifeCycle()).getComponentContext().activateEndpoint(serviceName, endpointName);
        } else {
            processor = createConsumerExchangeHandler(); 
            if (processor == null) {
               log.log
                  (Level.WARNING,
                   messages.getString("CJBIRT-W00887.noConsumerExchangeHandlerForEndpoint", this));
            } else {
                if (log.isLoggable(Level.FINER)) {
                    log.log
                        (Level.FINER,
                        "Setting the Delivery Channel on the Consumer Exchange Handler");
                }
               DeliveryChannel tempChannel =
                  ((AbstractComponentLifeCycle) serviceUnit.getComponent().getLifeCycle()).getComponentContext().getDeliveryChannel();
               if (tempChannel == null) {
                  log.log(Level.WARNING,messages.getString("CJBIRT-W00890.deliveryChannelIsNULL"));
               }
               processor.setDeliveryChannel(tempChannel);
            }
        }
    }

    public void deactivate() throws Exception {
        log.log(Level.INFO, messages.getString("CJBIRT-R00892.deactivatingEndpoint", this));
        processor = null;
        if (Role.PROVIDER.equals(role)) {
            ServiceEndpoint tmp = activatedEndpoint;
            activatedEndpoint = null;
            ((AbstractComponentLifeCycle) serviceUnit.getComponent().getLifeCycle()).getComponentContext().deactivateEndpoint(tmp);
        }
    }
    
    public AbstractMessageExchangeHandler getProcessor() {
        if (Role.PROVIDER.equals(role)) {
            return createProviderExchangeHandler();
        }
        return processor;
    }
    
}
